Std.ui.module("SwitchPanel",{
    /*[#module option:parent]*/
    parent:"widget",
    /*[#module option:option]*/
    option:{
        defaultClass:"StdUI_SwitchPanel",
        level:4,
        contentPadding:0,
        selected:0,
        items:null,
    },
    /*[#module option:events]*/
    events:"select",
    /*[#module option:extend]*/
    extend:{
        /*
         * extend render
         */
        render:function(){
            var that = this;
            var opts = that.opts;

            that._items.items(function(i,item){
                item.render();
            });
            that.select(opts.selected);

            if(opts.value){
                that.value(opts.value);
            }
        },
        /*
         * width
         */
        width:function(){
            var that         = this;
            var DOMMap       = that.DOMMap;
            var contentWidth = that.computeContentWidth();

            DOMMap.contents.width(contentWidth);
            that._items.items(function(i,item){
                item.width(contentWidth);
            });
        },
        /*
         * extend height
         */
        height:function(){
            var that          = this;
            var DOMMap        = that.DOMMap;
            var contentHeight = that.computeContentHeight();

            DOMMap.contents.height(contentHeight);
            that._items.items(function(i,item){
                item.height(contentHeight);
            });
        },
        /*
         * extend destroy
        */
        destroy:function()
        {
            var that = this;

            that._items.items(function(i,item){
                item.destroy();
            });
            that._items.clear();
            that._activeItem = null;
        },
        /*
         * extend remove
        */
        remove:function(index)
        {
            var that = this;
            var item = null;

            if(isObject(index)){
                index = that._items.indexOf(index);
            }
            if(isNumber(index) || isString(index))
            {
                if((item = that._items.items(index))){
                    item.destroy();
                    that._items.remove(index);
                }
            }
        },
        /*
         * enable
         */
        enable:function(enable){
            var that = this;

            that._items.items(function(i,item){
                item.enable(enable);
            });
        }
    },
    /*[#module option:protected]*/
    protected:{
        /*
         * create content
         */
        createContent:function(data)
        {
            var that    = this;
            var content = null;

            if(isString(data)){
                content = Std.ui("widget",{
                    tabIndex:null,
                    defaultClass:"StdUI_SwitchPanel_Content",
                    html:data
                });
            }else if(isWidget(data)){
                content = data;
                content.addClass("StdUI_SwitchPanel_Content");
            }else if(Std.is.DOM(data)){
                content = Std.ui("widget",{
                    tabIndex:null,
                    defaultClass:"StdUI_SwitchPanel_Content",
                    layout:{
                        ui:"VArrayLayout",
                        items:[data]
                    }
                });
            }else if(isLayout(data)){
                content = Std.ui("widget",{
                    tabIndex:null,
                    defaultClass:"StdUI_SwitchPanel_Content",
                    layout:data
                });
            }else if(isPlainObject(data)){
                content = Std.ui(data.ui || "widget",Std.extend({
                    tabIndex:null,
                    defaultClass:"StdUI_SwitchPanel_Content"
                },data));
            }
            return content;
        },
        /*
         * compute content width
        */
        computeContentWidth:function()
        {
            var that = this;
            var opts = that.opts;

            return that.width() - opts.contentPadding * 2 - that.boxSize.width - that.boxSize.extraWidth - 2;
        },
        /*
         * compute content height
        */
        computeContentHeight:function()
        {
            var that   = this;
            var opts   = that.opts;
            var height = that.height();

            return height - opts.contentPadding * 2 - 2;
        },
        /*
         * update Horizontal layout
        */
        updateLayout:function()
        {
            var that          = this;
            var DOMMap        = that.DOMMap;
            var contentWidth  = that.computeContentWidth();
            var contentHeight = that.computeContentHeight();

            DOMMap.contents.width(contentWidth);
            DOMMap.contents.height(contentHeight);

            that._items.items(function(i,item){
                item.width(contentWidth);
                item.height(contentHeight);
            });
            return that;
        }
    },
    /*[#module option:public]*/
    public:{
        /*
         * items
        */
        items:function(filter,arg2){
            return this._items.items(filter,arg2);
        },
        /*
         * item count
        */
        itemCount:function(){
            return this._items.length;
        },
        /*
         * content padding
         */
        contentPadding:function(padding){
            var that = this;

            that.DOMMap.contents.css("padding",padding + "px");

            return that;
        },
        /*
         * select
         */
        select:function(index,reference){
            var that = this;

            if(isString(index)){
                index = that.convertIndex(index,reference);
            }
            if(index !== -1 && index < that._items.length){
                that.activeIndex(index);
                that.emit("select",[index,that._items._list[index]],true);
            }
            return that;
        },
        /*
         * active index
         */
        activeIndex:function(index)
        {
            var that  = this;
            var items = that._items;

            if(index === undefined){
                return items.indexOf(that._activeItem);
            }
            if(items._list[index]){
                that.activeItem(items._list[index]);
            }
            return that;
        },
        /*
         * active Item
        */
        activeItem:function(item)
        {
            var that  = this;
            var opts  = that.opts;
            var items = that._items;

            if(item === undefined)
            {
                var index = that.activeIndex();

                if(index == -1){
                    return null;
                }
                return items._list[index];
            }
            var activeItem = that._activeItem;

            if(activeItem){
                activeItem.removeClass("_visible");
            }
            item.addClass("_visible");
            that._activeItem = item;

            if(!item.rendered){
                item.height(that.computeContentWidth());
                item.height(that.computeContentHeight());
                item.render();
            }
            return that;
        },
        /*
         * prepend
        */
        prepend:function(name,data){
            return this.insert(name,data,0);
        },
        /*
         * insert
        */
        insert:function(name,data,index)
        {
            var that   = this;
            var result = that._items.insert(name,data,index);

            if(that.rendered) {
                that.updateLayout();
            }
            return result;
        },
        /*
         * append
        */
        append:function(name,data)
        {
            var that   = this;
            var result = !isArray(name) ? that._items.append(name,data) : Std.each(name,function(i,data){
                return that._items.append(data);
            },true);

            if(that.rendered) {
                that.updateLayout();
            }
            return result;
        },
    },
    /*[#module option:main]*/
    main:function(that,opts)
    {
        that.DOMMap = {
            contents: newDiv("_contents").appendTo(that[0])
        };
        that._items = Std.items(function(item,attrs)
        {
            var content = that.createContent(item);

            that.DOMMap.contents.insert(content[0],attrs.index);

            if(that.rendered){
                content.width(that.computeContentWidth());
                content.width(that.computeContentHeight());
                content.render();
            }
            return content;
        });
        if(opts.items !== null){
            that.append(opts.items);
        }
        that.call_opts({
            contentPadding:0
        },true);
    },
});