/**
 * Std UI Kit Library
 * http://ui.stdjs.com
 * module: Videos
 */
Std.ui.module("Videos",{
    /*[#module option:parent]*/
    parent:"widget",
    /*[#module option:events]*/
    events:"meta attach detach mouseenter mouseleave error download scroll item",
    /*[#module option:option]*/
    option:{
        level:4,
        defaultClass:"StdUI_Videos",
        items:null,
        muted:"auto",
        playMode:"hover",  //hover, auto
        lazy:true
    },
    /*[#module option:private]*/
    private:{
        /*
         * timer
        */
        timer:null
    },
    /*[#module option:extend]*/
    extend:{
        /*
         * render
        */
        render:function()
        {
            var that = this;

            that[0].on("scroll",function(){
                that.refreshVideos();
            });
        },
        /*
         * destroy
        */
        destroy:function()
        {
            var that = this;

            that.clear();
        },
        /*
         * height
        */
        height:function(n)
        {
            if(isNumber(n) && this.rendered){
                this.updateRectSize();
                this.refreshVideos();
            }
        },
        /*
         * width
        */
        width:function(n)
        {
            if(isNumber(n) && this.rendered){
                this.updateRectSize();
                this.refreshVideos();
            }
        },
        /*
         * remove
        */
        /*
        remove:function(index)
        {
            var that = this;
            var item = null;

            if(index !== undefined && (item = that._items.items(index))){
                that[0].remove(item.DOMMap.item);
                that._items.remove(index);
            }
        }*/
    },
    /*[#module option:initialize]*/
    initialize: {
        /*
         * init events
        */
        init_events:function()
        {
            var that = this;
            var opts = that.opts;

            that[0].on("mouseenter","> ._item > ._frame",function(e)
            {
                var index = this.parent().index();
                var item  = that._items.items(index);

                if(opts.playMode === "hover"){
                    that.play(item);
                }
                if(item.states.attached && opts.muted === "auto"){
                    item.DOMMap.video.dom.muted = false;
                }
                that.emit("mouseenter",item);
            });
            that[0].on("mouseleave","> ._item > ._frame",function(e)
            {
                var index = this.parent().index();
                var item  = that._items.items(index);

                if(item.states.attached)
                {
                    if(opts.muted === "auto"){
                        item.DOMMap.video.dom.muted = true;
                    }
                    if(opts.playMode === "hover" && !that._states.playing_all){
                        that.pause(item);
                    }
                }
                that.emit("mouseleave",item);
            });

            that[0].on("click","> ._item > ._actions",function(e)
            {
                var index = this.parent().index();
                that.showActions(index);
            });
        }
    },
    /*[#module option:protected]*/
    protected:{
        /*
         * attach
        */
        attach:function(item)
        {
            var that = this;
            var opts = that.opts;

            if(item.states.attached || !that.attachable(item.index)){
                return;
            }
            item.states.attached = true;
            that.createVideo(item);

            if(!item.data.url){
                return item.DOMMap.video.emit("error",item);
            }
            that.emit("attach",item)

            if(item.DOMMap.video.dom.readyState >= 2){
                item.DOMMap.video.emit("loadedmetadata",item);
            }
            if(opts.playMode === "auto" || that._states.playing_all){
                that.play(item)
            }
        },
        /*
         * detach
        */
        detach:function(item,drawPoster)
        {
            var that = this;
            var opts = that.opts;

            if(!item.states.attached){
                return;
            }
            if(!item.DOMMap.canvas){
                item.DOMMap.canvas = newDom("canvas","_canvas");
                item.DOMMap.frame.append(item.DOMMap.canvas);
            }
            if(drawPoster !== false){
                that.drawPoster(item);
            }
            that.pause(item);
            that.emit("detach",item);
            item.states.attached   = false;
            item.states.metaloaded = false;
            item.states.canplay    = false;

            item.DOMMap.frame.remove(item.DOMMap.video);
            item.DOMMap.canvas.show();
            item.DOMMap.duration.addClass("_visible");
            item.DOMMap.video.off();
            item.DOMMap.video = null;
        },
        /*
         * refresh videos
        */
        refreshVideos:function(callback)
        {
            var that = this;
            var opts = that.opts;

            if(opts.lazy === false || !that._states.has_rect){
                return;
            }
            if(that._scroll_timer !== null){
                clearTimeout(that._scroll_timer);
                that._scroll_timer = null;
            }
            that._scroll_timer = setTimeout(function()
            {
                var scrollTop = that[0].scrollTop();
                var columns   = Math.floor(that._rect.width / that._rect.itemWidth);
                var begin	  = Math.floor(scrollTop / that._rect.itemHeight) * columns;
                var end  	  = Math.ceil((scrollTop + that._rect.height) / that._rect.itemHeight) * columns;

                that._states.begin		= begin || 0;
                that._states.end 		= end || 0;
                that._states.scroll_top = scrollTop;
                that._scroll_timer 	    = null;
                that._visuals.each(function(i,item){
                    if(item.index !== -1 && !that.attachable(item.index)){
                        that.detach(item);
                    }
                })
                that._visuals.length = 0;

                for(var i=begin,length=Math.min(end,that._items.length);i<length;++i)
                {
                    var item = that._items.items(i);

                    if(opts.playMode === "hover" && !that._states.playing_all){
                        that.attach(item);
                    }else{
                        that.play(item);
                    }
                    that._visuals.push(item);
                }
                callback && callback();
                that.emit("scroll")
            },300);
        },
        /*
         * update meta text
        */
        updateMetaText:function(item)
        {
            var video 	   = item.DOMMap.video;
            var resolution = "";
            var width 	   = video.dom.videoWidth;
            var height 	   = video.dom.videoHeight;

            if(item.data.type){
                resolution = item.data.type;
            }
            let p = item.states.p = Math.min(width,height);

            if(p < 480){
                item.DOMMap.quality.addClass("_low");
            }else if(p >= 720){
                item.DOMMap.quality.addClass("_HD");
            }
            item.DOMMap.quality.show().html(p + "P");
            item.DOMMap.resolution.html(resolution + " " + width + "x" + height);

            if(item.data.duration !== 0){
                let minutes = Math.floor(item.states.duration / 60);
                let seconds = Math.floor(item.states.duration - minutes * 60);

                item.DOMMap.duration.html(minutes + ":" + seconds);
            }
        },
        /*
         * update rect size
        */
        updateRectSize:function()
        {
            var that = this;
            var opts = that.opts;

            if(that._items.length === 0){
                return;
            }
            var first  = that._items.items(0).DOMMap.item;
            var margin = first.css("margin") * 2;
            var width  = first.outerWidth() + margin;
            var height = first.outerHeight() + margin;

            that._states.has_rect = true;
            that._rect = {
                itemWidth:width,
                itemHeight:height,
                width:that[0].clientWidth(),
                height:that[0].clientHeight(),
            }
            if(!opts.lazy){
                that._items.items(function(i,item){
                    that.attach(item);
                });
            }
        },
        /*
         * draw poster
        */
        drawPoster:function(item)
        {
            if(!item.states.canplay){
                return;
            }
            var canvas	 	= item.DOMMap.canvas.dom;
            var ctx 		= canvas.getContext("2d");
            var frameWidth  = item.DOMMap.frame.width();
            var frameHeight = item.DOMMap.frame.height();
            var videoRatio  = item.states.width / item.states.height;
            var drawWidth   = videoRatio > frameWidth / frameHeight ? frameWidth : frameHeight * videoRatio;
            var drawHeight  = videoRatio > frameWidth / frameHeight ? drawWidth / videoRatio : frameHeight;
            var x 		    = (frameWidth - drawWidth) / 2;
            var y 		    = (frameHeight - drawHeight) / 2;

            canvas.width = frameWidth;
            canvas.height = frameHeight;

            ctx.clearRect(0,0,frameWidth,frameHeight);
            ctx.drawImage(item.DOMMap.video.dom,x,y,drawWidth,drawHeight);

            /*
            canvas.width  = item.states.width;
            canvas.height = item.states.height;
            ctx.drawImage(item.DOMMap.video.dom,0,0,item.states.width,item.states.height);
            */
            //console.log(canvas.toDataURL("image/png"))
            //item.DOMMap.frame.css("background",canvas.toDataURL("image/png"));
        },
        /*
         * create video
        */
        createVideo:function(item)
        {
            var that  = this;
            var opts  = that.opts;
            var video = newDom("video","_video").attr({
                controls:"controls"
            });
            item.DOMMap.video = video;

            video.dom.muted = opts.muted === "auto" || opts.muted === true;
            video.on("error",function(){
                if(!item.states.metaloaded){
                    item.states.error = true;
                }
                item.DOMMap.error.show();
                that.emit("error",item);
            });
            video.on("canplay",function(){
                item.states.canplay = true;
            });
            video.on("loadedmetadata",function(e)
            {
                item.states.width      = video.dom.videoWidth;
                item.states.height     = video.dom.videoHeight;
                item.states.duration   = video.dom.duration;
                item.states.metaloaded = true;
                item.DOMMap.canvas && item.DOMMap.canvas.hide();
                item.DOMMap.duration.removeClass("_visible");

                if(item.states.seek !== 0){
                    video.dom.currentTime = item.states.seek;
                }
                that.updateMetaText(item);
                that.emit("meta",item);
            });
            video.on("timeupdate",function(e) {
                item.states.seek = video.dom.currentTime;
            });

            item.DOMMap.frame.append(item.DOMMap.video);

            return item.DOMMap.video;
        },
        /*
         * create item
        */
        createItem:function(data,index)
        {
            var that = this;
            var opts = that.opts;
            var item =
                {
                    DOMMap:{
                        item:newDiv("_item"),
                        frame:newDiv("_frame"),
                        video:null,
                        canvas:null,
                        resolution:newDiv("_resolution"),
                        quality:newDiv("_quality"),
                        duration:newDiv("_duration"),
                        error:newDiv("_error"),

                        information:newDiv("_information"),
                        file_name:newDiv("_row _file_name"),
                        file_name_text:newDom("span","_text"),
                        page_url:newDom("a","_row _page_url").attr("target","_blank"),

                        actions:newDiv("_actions")
                    },
                    states:{
                        attached: false,
                        metaloaded: false,
                        canplay: false,
                        error: false,
                        seek: 0,
                        duration: 0,
                        width: 1,
                        height: 1,
                        p:0
                    },
                    data:data,
                    index:index,

                    icon:function(url)
                    {
                        item.DOMMap.page_url.css({
                            textIndent:"18px",
                            backgroundImage:"url('" + url + "')"
                        })
                    }
                };
            item.DOMMap.frame.append([item.DOMMap.resolution,item.DOMMap.quality,item.DOMMap.duration,item.DOMMap.error]);
            item.DOMMap.information.append([item.DOMMap.file_name,item.DOMMap.page_url]);
            item.DOMMap.item.append([item.DOMMap.frame,item.DOMMap.information,item.DOMMap.actions]);

            if(!isObject(data)){
                return item;
            }
            item.DOMMap.file_name.append(item.DOMMap.file_name_text);
            item.DOMMap.file_name_text.on("click",function(){
                that.emit("download",item);
            });
            if(data.text){
                item.DOMMap.resolution.html(data.text);
            }else if(data.type){
                item.DOMMap.resolution.html(data.type);
            }
            if(data.file_name){
                item.DOMMap.file_name_text.html(data.file_name).attr("title",data.file_name);
            }else if(data.text){
                item.DOMMap.file_name_text.html(data.text).attr("title",data.text);
            }
            if(data.page_url){
                item.DOMMap.page_url.html(data.page_url).attr("title",data.page_url).attr("href",data.page_url);
            }
            if(data.icon){
                item.icon(data.icon);
            }
            that.emit("item",item);

            return item;
        },
        /*
         * show actions
        */
        showActions:function(index)
        {
            var that   = this;
            var item   = that._items.items(index);
            var offset = {
                x:item.DOMMap.actions.offsetLeft(),
                y:item.DOMMap.actions.offsetTop(),
            };
            if(!that.opts.actions){
                return;
            }
            that._actionsMenu = Std.ui("Menu",{
                items:function()
                {
                    var items = [];

                    Std.each(that.opts.actions,function(i,action)
                    {
                        items.push({
                            text:action.text,
                            icon:action.icon,
                            click:function(){
                                action.click && action.click(item);
                                that._actionsMenu.remove();
                                that._actionsMenu = null
                            }
                        });
                    });
                    return items;
                }(),
                renderTo:item.DOMMap.item,
                css:{
                    position:"absolute",
                    margin:0
                }
            });
            that._actionsMenu.visible(true);
            that._actionsMenu.move(
                offset.x + (item.DOMMap.actions.outerWidth() - that._actionsMenu.width()),
                offset.y + (item.DOMMap.actions.outerHeight() - that._actionsMenu.height())
            );
            that._timer = setTimeout(function()
            {
                Std.dom(document).once("mousedown",function(){
                    if(that._actionsMenu){
                        that._actionsMenu.remove();
                        that._actionsMenu = null;
                    }
                });
            },1);
        }
    },
    /*[#module option:public]*/
    public:{
        /*
         * count
        */
        count:function()
        {
            return this._items.length;
        },
        /*
         * attachable
        */
        attachable:function(index)
        {
            return this.rendered && (!this.opts.lazy || (index >= this._states.begin && index < this._states.end));
        },
        /*
         * items
        */
        items:function(filter,arg2){
            return this._items.items(filter,arg2);
        },
        /*
         * insert cell
        */
        insert:function(name,data,index)
        {
            var that = this;
            var opts = that.opts;
            var item = null;

            if(!isNumber(index) || index === -1 || index > that._items.length){
                index = that._items.length;
            }
            that._items.insert(name,data,index,function(data){
                return item = that.createItem(data,index);
            });
            if(item === null){
                return false;
            }
            that[0].insert(item.DOMMap.item,index);

            if(!that._states.has_rect && that.rendered){
                that.updateRectSize();
                that.refreshVideos();
            }
            if(item.data.url){
                that.attach(item);
            }
            return that;
        },
        /*
         * append
        */
        append:function(name,data)
        {
            var that = this;

            if(isArray(name)){
                Std.each(name,function(i,data){
                    that.insert(data);
                })
            }else{
                that.insert(name,data);
            }
            return that;
        },
        /*
         * play
        */
        play:function(item)
        {
            var that = this;

            if(isNumber(item)){
                item = this._items.items(item);
            }
            if(isObject(item) && item.DOMMap)
            {
                that.attach(item);

                if(item.DOMMap.video && item.DOMMap.video.dom.paused){
                    item.DOMMap.video.dom.play().then(() => {}).catch((e) => {});
                }
            }else if(item === undefined){
                that._states.playing_all = true;

                if(that.opts.lazy){
                    that._visuals.each(function(i,item){
                        that.play(item);
                    });
                }else{
                    that._items.items(function(i,item){
                        that.play(item);
                    });
                }
            }
        },
        /*
         * play
        */
        pause:function(item)
        {
            var that = this;

            if(isNumber(item)){
                item = this._items.items(item);
            }
            if(isObject(item) && item.DOMMap)
            {
                if(item.states.attached && !item.DOMMap.video.dom.paused){
                    item.DOMMap.video.dom.pause();
                }
            }else if(item === undefined){
                that._states.playing_all = false;

                if(that.opts.lazy){
                    that._visuals.each(function(i,item){
                        that.pause(item);
                    });
                }else{
                    that._items.items(function(i,item){
                        that.pause(item);
                    });
                }
            }
        },
        /*
         * remove
        */
        remove:function(index)
        {
            var that = this;

            if(isNumber(index))
            {
                var item = that._items.items(index);

                if(isObject(item))
                {
                    that.detach(item,false);
                    item.DOMMap.item.remove();
                    item.index = -1;
                    that._items.remove(index);

                    for(let i=index;i<that._items.length;++i){
                        that._items.items(i).index = i;
                    }
                    that.refreshVideos();
                }
            }else if(isArray(index))
            {
                let temp = [];

                index.forEach(i =>
                {
                    if(isNumber(i) && i < that._items.length){
                        temp.push(i);
                    }
                });
                temp.sort(function(a,b){
                    return b - a;
                });
                if(temp.length !== 0)
                {
                    for(let i=0;i<temp.length;++i)
                    {
                        var item = that._items.items(temp[i]);
                        that.detach(item,false);
                        item.DOMMap.item.remove();
                        that._items.remove(temp[i]);
                        item.index = -1;
                    }
                    for(let i=temp.last();i<that._items.length;++i){
                        that._items.items(i).index = i;
                    }
                    that.refreshVideos();
                }
            }
        },
        /*
         * clear
        */
        clear:function()
        {
            var that = this;

            that._items.items(function(i,item){
                if(that[0]){
                    that.detach(item,false);
                    item.DOMMap.item.remove();
                }
                item.index = -1;
            });
            if(that._actionsMenu){
                that._actionsMenu.remove();
                that._actionsMenu = null;
            }
            that._items.clear();
            that._visuals.length 	= 0;
            that._states.scroll_top = 0;
            that._states.begin 	    = 0;
            that._states.end 		= 0;
        }
    },
    /*[#module option:main]*/
    main:function(that,opts)
    {
        that._items   	  = Std.items();
        that._rect    	  = {};
        that._visuals 	  = [];
        that._actionsMenu = null;
        that._states  	  = {
            has_rect: false,
            playing_all: false,
            loading_all: false,
            scroll_top: 0,
            begin:0,
            end:0
        }
        if(opts.items){
            that.append(opts.items);
        }
        that.init_events();
    }
});

